#include "pme_internalDataStructure.h"
#include "gcd_cpm.h"



int pbc_int(int i, int N)
{
	if (i >= N)
		i -= N;
	else if (i < 0)
		i += N;
	return i;
}

real ***CreateCubic_pme(int rows, int cols, int lays)
{
	int i, j;
	real ***m;
	m = (real ***)fftw_malloc(sizeof(real **) * rows);
	for (i = 0; i < rows; i++)
	{
		m[i] = (real **)fftw_malloc(sizeof(real *) * cols);
		for (j = 0; j < cols; j++)
			m[i][j] = (real *)fftw_malloc(sizeof(real) * lays);
	}
	return m;
}

real *CreateCubic2(int rows, int cols, int lays)
{
	real *m;
	m = (real *)fftw_malloc(sizeof(real) * rows * cols * lays);
	return m;
}

real *CreateMatrix2(int rows, int cols)
{
	real *m;
	m = (real *)fftw_malloc(sizeof(real) * rows * cols);
	return m;
}

real *CreateVector2(int rows)
{
	real *m;
	m = (real *)fftw_malloc(sizeof(real) * rows);
	return m;
}

double *CreateVector2Double(int rows)
{
	double *m;
	m = (double *)fftw_malloc(sizeof(double) * rows);
	return m;
}

float *CreateVector2Float(int rows)
{
	float *m;
	m = (float *)fftw_malloc(sizeof(float) * rows);
	return m;
}

int *CreateVector2Int(int rows)
{
	int *m;
	m = (int *)fftw_malloc(sizeof(int) * rows);
	return m;
}




real getPhiKOfElectrode(gmx_pme_t pme, real *grid,
						pme_atomcomm_t *atc, int atomID)
{
	splinedata_t *spline;
	int n, ithx, ithy, ithz, i0, j0, k0;
	int index_x, index_xy;
	int *idxptr;
	real pot, tx, ty, qn, gval;
	real *thx, *thy, *thz;
	int norder;
	int order;

	spline = &atc->spline[0];
	//printf("spline->n=%d\n",spline->n);

	order = pme->pme_order;

	n = atomID;
	qn = atc->q[n];

	//printf("qn = %f \n",qn);

	idxptr = atc->idx[n];
	norder = n * order;

	i0 = idxptr[XX];
	j0 = idxptr[YY];
	k0 = idxptr[ZZ];
	//printf("i0=%d,j0=%d,k0=%d\n",i0,j0,k0);
	/* Pointer arithmetic alert, next three statements */
	thx = spline->theta[XX] + norder;
	thy = spline->theta[YY] + norder;
	thz = spline->theta[ZZ] + norder;

	pot = 0;
	for (ithx = 0; (ithx < order); ithx++)
	{
		index_x = (i0 + ithx) * pme->pmegrid_ny * pme->pmegrid_nz;
		tx = thx[ithx];

		for (ithy = 0; (ithy < order); ithy++)
		{
			index_xy = index_x + (j0 + ithy) * pme->pmegrid_nz;
			ty = thy[ithy];

			for (ithz = 0; (ithz < order); ithz++)
			{
				gval = grid[index_xy + (k0 + ithz)];
				//printf("tx=%f,ty=%f,tz=%f,=%f\n",tx,ty,thz[ithz],gval);
				pot += tx * ty * thz[ithz] * gval;
			}
		}
	}

	return pot;
}

void getPhiOfElectrode(gmx_pme_t pme, real *grid, pme_atomcomm_t *atc, pme_atomcomm_t *atcForSpline, real rcutoff, matrix box, real ewaldcoeff, int NUserSelectGrid,
					   real *UserSelectPosition, real *UserSetPot, real *externElectrodePot, real *externElectrodeReal, real *externElectrodeRec)
{
	static int natoms, order;
	static int nx, ny, nz, pnx, pny, pnz;
	static real spacing_x, spacing_y, spacing_z, boxX, boxY, boxZ, eta, rlist;
	static int bFirst = 1;

	int i, j, i0, j0, k0, pmeidx;
	int *idxptr;
	real qn, xnX, xnY, xnZ, deltaX, deltaY, deltaZ, rij;
	static real *PotRec, *PotReal, *Pot3dc, *PotRealTemp;

	if (bFirst)
	{
		//printf("g_ewald = %f\n",ewaldcoeff);
		order = pme->pme_order;
		natoms = atc->n;
		nx = pme->nkx;
		ny = pme->nky;
		nz = pme->nkz;
		pnx = pme->pmegrid_nx;
		pny = pme->pmegrid_ny;
		pnz = pme->pmegrid_nz;
		spacing_x = box[XX][XX] / nx;
		spacing_y = box[YY][YY] / ny;
		spacing_z = box[ZZ][ZZ] / nz;
		boxX = box[XX][XX];
		boxY = box[YY][YY];
		boxZ = box[ZZ][ZZ];
		eta = 19.79;
		rlist = rcutoff * 1.2;
		PotRec = CreateVector2(NUserSelectGrid);
		PotReal = CreateVector2(NUserSelectGrid);
		Pot3dc = CreateVector2(NUserSelectGrid);
		PotRealTemp = CreateVector2(  NUserSelectGrid);
		bFirst = 0;
	}

	for (i = 0; i <   NUserSelectGrid; i++)
	{
		PotRealTemp[i] = 0.0;
	}

	for (i = 0; i < NUserSelectGrid; i++)
	{
		PotReal[i] = 0.0;
		PotRec[i] = 0.0;
	}

	int Nj;
	/* get phi_r on the UserSelectPosition */
	for (i = NUserSelectGrid; i < natoms; i++)
	{
		qn = atc->q[i];
		xnX = atc->x[i][XX];
		xnY = atc->x[i][YY];
		xnZ = atc->x[i][ZZ];

		if ((qn != 0))
		{
			Nj = 0;
			for (j = 0; j < NUserSelectGrid; j++)
			{
				deltaX = fabsf(xnX - UserSelectPosition[3 * j]);
				if (deltaX > (boxX / 2))
				{
					deltaX = boxX - deltaX;
				}
				deltaY = fabsf(xnY - UserSelectPosition[3 * j + 1]);
				if (deltaY > (boxY / 2))
				{
					deltaY = boxY - deltaY;
				}
				deltaZ = fabsf(xnZ - UserSelectPosition[3 * j + 2]);
				/* 3D periodic boundary conditionn */
				if (deltaZ > (boxZ / 2))
				{
					deltaZ = boxZ - deltaZ;
				}
				if (deltaX < rlist && deltaY < rlist && deltaZ < rlist)
				{
					rij = sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);
					if (rij < rlist)
					{
						Nj++;
					}
					if (rij < rcutoff)
					{
						PotRealTemp[j] += ONE_4PI_EPS0 / pme->epsilon_r * qn * (erfc(ewaldcoeff * rij) / rij - erfc(eta * rij) / rij);
					}
				}
			}
		}
	}


	/* get phi_k on the UserSelectPosition */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		PotRec[i] = getPhiKOfElectrode(pme, grid, atcForSpline, i);
		/* debug */
		if (0)
		{
			printf("i=%d,potrec=%f\n", i, potUnit * PotRec[i]);
		}
	}

	/* get phi_3dc if necessary */
	if (ThreedOrThreedc == 1)
	{
		for (i = 0; i < NUserSelectGrid; i++)
		{
			Pot3dc[i] = (UserSelectPosition[3 * i + 2] - boxZ / 2) * ElectricField;
			//printf("Pot3dc[%d]=%f\n",i,Pot3dc[i]);
		}
	}

	for (i = 0; i < NUserSelectGrid; i++)
	{
			PotReal[i] += PotRealTemp[ i];
			//printf("PotRealTemp[%d*2+%d]=%f",j,i,PotRealTemp[j*NUserSelectGrid+i]);
		if (ThreedOrThreedc == 1)
		{
			//externElectrodePot[i] = UserSetPot[i]-(PotReal[i]+PotRec[i]+Pot3dc[i])*potUnit;
			externElectrodePot[i] = -(PotReal[i] + PotRec[i] + Pot3dc[i]) * potUnit;
			externElectrodeReal[i] = PotReal[i];
			externElectrodeRec[i] = PotRec[i] + Pot3dc[i];
		}
		else
		{
			//externElectrodePot[i] = UserSetPot[i]-(PotReal[i]+PotRec[i])*potUnit;
			externElectrodePot[i] = -(PotReal[i] + PotRec[i]) * potUnit; // lzeng for cc
			externElectrodeReal[i] = PotReal[i];
			externElectrodeRec[i] = PotRec[i];
		}
		/* debug */
		if (0)
		{
			printf("x=%f, y=%f, z=%f, potreal=%f, potrec=%f, pot3dc=%f, ElectrodePot=%f\n", UserSelectPosition[3 * i], UserSelectPosition[3 * i + 1], UserSelectPosition[3 * i + 2], potUnit * PotReal[i], potUnit * PotRec[i], potUnit * Pot3dc[i], externElectrodePot[i]);
		}
	}
}

void getChargeOfElectrode(gmx_pme_t pme, int NUserSelectGrid, float *inv_MatrixA, real *externElectrodePot, double *externElectrodeCharge, real *chargeA, real *Phiofelectrode_cc)
{
	static double *ElectrodeCharge;
	static double *ElectrodeChargeplus2;
	static double *VectorBplus2; /* lzneg for cc
	/* itpbs 20180119, charge tune */
	static double *dElectrodeCharge1V, sumCharge1V = 0.0;
	static int bFirst = 1;
	int i, ii;
	real nowQset;				   // itplz
	static float *Usersetcurrents; //lzeng 12/23/2019 for current
	static int index_current = 0;
	static int numofcurrents;
	if (bFirst)
	{
		ElectrodeCharge = CreateVector2Double(NUserSelectGrid);
		ElectrodeChargeplus2 = CreateVector2Double(NUserSelectGrid + 2);
		VectorBplus2 = CreateVector2Double(NUserSelectGrid + 2);
		bFirst = 0; //
		char CPMtemp[36];
		int idxx;
	}

	/* initialize */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		ElectrodeCharge[i] = 0.0;
	}
	for (i = 0; i < NUserSelectGrid + 2; i++)
	{
		ElectrodeChargeplus2[i] = 0.0;
		VectorBplus2[i] = 0.0;
	}
	for (i = 0; i < NUserSelectGrid; i++)
	{
		VectorBplus2[i] = externElectrodePot[i];
	}
	// itplz add nowQ
	nowQset = index_current % GCDPstep;
	if (nowQset < (GCDPstep / 2))
		nowQset = GCDQ * nowQset / (GCDPstep / 2);
	else
		nowQset = 2 * GCDQ - GCDQ * nowQset / (GCDPstep / 2);
	VectorBplus2[NUserSelectGrid] = -1 * nowQset / evscale;
	VectorBplus2[NUserSelectGrid + 1] = nowQset / evscale;
	index_current++;

	/* Matrix multiple */
	for (i = 0; i < NUserSelectGrid + 2; i++)
	{
		for (ii = 0; ii < NUserSelectGrid + 2; ii++)
		{
			ElectrodeChargeplus2[i] = ElectrodeChargeplus2[i] + inv_MatrixA[i * (2 + NUserSelectGrid) + ii] * VectorBplus2[ii];
			//printf("%d %d %f %f \n",i,i*NUserSelectGrid+ii,inv_MatrixA[i*NUserSelectGrid+ii],externElectrodePot[ii]);
		}
	}
	Phiofelectrode_cc[0] = ElectrodeChargeplus2[NUserSelectGrid];
	Phiofelectrode_cc[1] = ElectrodeChargeplus2[NUserSelectGrid + 1];
	if (0)
	{
		printf("potential of electrode %f %f \n", ElectrodeChargeplus2[NUserSelectGrid], ElectrodeChargeplus2[NUserSelectGrid + 1]);
	}
	for (i = 0; i < NUserSelectGrid; i++)
	{
		ElectrodeCharge[i] = ElectrodeChargeplus2[i];
	}

	/* transfer to extern variables */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		externElectrodeCharge[i] = evscale * ElectrodeCharge[i];
		chargeA[i] = evscale * ElectrodeCharge[i];
		//printf("%d \t %f\n",i,chargeA[i]);
	}
}
// itplz
void force_callz(gmx_pme_t pme, real *grid, pme_atomcomm_t *atc, real rcutoff, matrix box, real ewaldcoeff, int NUserSelectGrid,
				 real *UserSelectPosition, real *UserSetPot, real *externElectrodePot, real *externElectrodeReal, real *externElectrodeRec, real *chargeA)
{
	static int natoms, order;
	static int nx, ny, nz, pnx, pny, pnz;
	static real spacing_x, spacing_y, spacing_z, boxX, boxY, boxZ, alpha, alpha2, alpha3, eta, eta2, eta3, rcutoff2, sqrtM_PI;
	static int bFirst = 1;

	int i, j, i0, j0, k0, pmeidx, index;
	int *idxptr;
	real qn, qelectrode, xnX, xnY, xnZ, deltaX, deltaY, deltaZ, rij, rij2, rij3, fij_r;

	if (bFirst)
	{
		printf("g_ewald = %f\n", ewaldcoeff);
		order = pme->pme_order;
		natoms = atc->n;
		nx = pme->nkx;
		ny = pme->nky;
		nz = pme->nkz;
		pnx = pme->pmegrid_nx;
		pny = pme->pmegrid_ny;
		pnz = pme->pmegrid_nz;
		spacing_x = box[XX][XX] / nx;
		spacing_y = box[YY][YY] / ny;
		spacing_z = box[ZZ][ZZ] / nz;
		boxX = box[XX][XX];
		boxY = box[YY][YY];
		boxZ = box[ZZ][ZZ];
		alpha = ewaldcoeff;
		alpha2 = alpha * alpha;
		alpha3 = alpha * alpha * alpha;
		eta = 19.79;
		eta2 = eta * eta;
		eta3 = eta * eta * eta;
		rcutoff2 = rcutoff * rcutoff;
		sqrtM_PI = sqrt(M_PI);
		bFirst = 0;
	}

	for (i = NUserSelectGrid; i < natoms; i++)
	{
			qn = atc->q[i];
			xnX = atc->x[i][XX];
			xnY = atc->x[i][YY];
			xnZ = atc->x[i][ZZ];

			if ((qn != 0))
			{
				//printf("the previous force on %d atom is x=%f\t y=%f\t z=%f\n",i,atc->f[i][XX], atc->f[i][YY], atc->f[i][ZZ]);
				for (j = 0; j < NUserSelectGrid; j++)
				{
					qelectrode = chargeA[j];
					deltaX = xnX - UserSelectPosition[3 * j];
					if (fabsf(deltaX) > (boxX / 2))
					{
						if (deltaX < 0)
							deltaX = boxX + deltaX;
						else
							deltaX = -boxX + deltaX;
					}
					deltaY = xnY - UserSelectPosition[3 * j + 1];
					if (fabsf(deltaY) > (boxY / 2))
					{
						if (deltaY < 0)
							deltaY = boxY + deltaY;
						else
							deltaY = -boxY + deltaY;
					}
					deltaZ = xnZ - UserSelectPosition[3 * j + 2];
					if (fabsf(deltaZ) > (boxZ / 2))
					{
						if (deltaZ < 0)
							deltaZ = boxZ + deltaZ;
						else
							deltaZ = -boxZ + deltaZ;
					}

					if (fabsf(deltaX) < rcutoff && fabsf(deltaY) < rcutoff && fabsf(deltaZ) < rcutoff)
					{
						rij2 = deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ;
						if (rij2 < rcutoff2)
						{
							rij = sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);
							rij3 = rij * rij2;
							//fij_r = ONE_4PI_EPS0/pme->epsilon_r*qn*qelectrode*(alpha3*pmecalcFSR(alpha2*rij2)-eta3*pmecalcFSR(eta2*rij2));
							fij_r = 2 * alpha * exp(-alpha2 * rij2) / (sqrtM_PI * rij2) + erfc(alpha * rij) / (rij3) - (2 * eta * exp(-eta2 * rij2) / (sqrtM_PI * rij2) + erfc(eta * rij) / (rij3));
							fij_r = ONE_4PI_EPS0 / pme->epsilon_r * qn * qelectrode * fij_r;
							atc->f[i][XX] += fij_r * deltaX;
							atc->f[i][YY] += fij_r * deltaY;
							atc->f[i][ZZ] += fij_r * deltaZ;
						}
					}
				}
				//printf("the force on %d atom is x=%f\t y=%f\t z=%f\n",i,atc->f[i][XX], atc->f[i][YY], atc->f[i][ZZ]);
			}
		
	}
}

void getPhiOfBulk(gmx_pme_t pme, real *grid, pme_atomcomm_t *atc, matrix box, real *boundary, double *externBulkPot)
{
	static int natoms, order;
	static int nx, ny, nz, pnx, pny, pnz, calnx, calny, calnz;
	static real spacing_x, spacing_y, spacing_z, boxX, boxY, boxZ;
	static int bFirst = 1;
	real xnZ;
	int boundary_low, boundary_top, loopN;
	int index, i0, j0, k0, pmeidx;
	double bulkpot;
	if (bFirst)
	{
		order = pme->pme_order;
		natoms = atc->n;
		nx = pme->nkx;
		ny = pme->nky;
		nz = pme->nkz;
		pnx = pme->pmegrid_nx;
		pny = pme->pmegrid_ny;
		pnz = pme->pmegrid_nz;
		spacing_x = box[XX][XX] / nx;
		spacing_y = box[YY][YY] / ny;
		spacing_z = box[ZZ][ZZ] / nz;
		boxX = box[XX][XX];
		boxY = box[YY][YY];
		boxZ = box[ZZ][ZZ];

		bFirst = 0;
	}
	boundary_low = (int)((boundary[0] + MK_INT) / spacing_z);
	boundary_top = (int)((boundary[1] + MK_INT) / spacing_z);
	calnx = nx;
	calny = ny;
	calnz = boundary_top - boundary_low + 1;
	if (calnz < 0)
	{
		printf("boundary_top %f is smaller than boundary_low %f\n", boundary[0], boundary[1]);
		exit(0);
	}
	loopN = calnx * calny * calnz;

	bulkpot = 0;

	for (index = 0; index < loopN; index++)
	{
		i0 = index / (calny * calnz); //
		j0 = index / calnz - i0 * calny;
		k0 = index % calnz;
		i0 = i0 + (order / 2 - 1);
		j0 = j0 + (order / 2 - 1);
		k0 = k0 + (order / 2 - 1) + boundary_low;
		xnZ = (k0 - (order / 2 - 1)) * spacing_z;
		pmeidx = (i0 * pny + j0) * pnz + k0;
		/* debug */
		if (0)
		{
			if (i0 == 1 && j0 == 1)
			{
				printf("i0=%d, j0=%d, k0=%d,grid[%d]=%f\n", i0, j0, k0, pmeidx, potUnit * grid[pmeidx]);
			}
		}
		if (ThreedOrThreedc == 1)
		{
			bulkpot += grid[pmeidx] + ElectricField * (xnZ - boxZ / 2); //
		}
		else
		{
			bulkpot += grid[pmeidx];
		}
	}
	*externBulkPot = bulkpot / loopN;
}

void getPhiOfElectrodeRerun(gmx_pme_t pme, int NUserSelectGrid, float *MatrixA, real *electrodeReal, real *electrodeRec,
							double *electrodeCharge, double *externAllElectrodePot, double *externAvePosPot, double *externAveNegPot)
{
	static double *allElectrodePot;
	static int bFirst = 1;
	double avePosPot = 0.0, aveNegPot = 0.0;
	int i, ii;
	if (bFirst)
	{
		allElectrodePot = CreateVector2Double(NUserSelectGrid);
		bFirst = 0; // bug 20170214. itpbs
	}

	/* initialize */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		allElectrodePot[i] = 0.0;
	}

	/* Matrix multiple */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		for (ii = 0; ii < NUserSelectGrid; ii++)
		{
			allElectrodePot[i] = allElectrodePot[i] + MatrixA[i * NUserSelectGrid + ii] * electrodeCharge[ii];
			//printf("%d %d %f %f \n",i,i*NUserSelectGrid+ii,MatrixA[i*NUserSelectGrid+ii],electrodeCharge[ii]);
		}
	}

	/* transfer to extern variables */
	for (i = 0; i < NUserSelectGrid; i++)
	{
		externAllElectrodePot[i] = allElectrodePot[i] / evscale + (electrodeReal[i] + electrodeRec[i]) * potUnit;
		//printf("%d \t %f \t %f \t %f \t %f\n",i,externAllElectrodePot[i], allElectrodePot[i]/evscale, electrodeReal[i]*potUnit, electrodeRec[i]*potUnit);
	}

	for (i = 0; i < (NUserSelectGrid / 2); i++)
	{
		avePosPot += externAllElectrodePot[i];
		aveNegPot += externAllElectrodePot[i + NUserSelectGrid / 2];
	}
	*externAvePosPot = avePosPot / (NUserSelectGrid / 2);
	*externAveNegPot = aveNegPot / (NUserSelectGrid / 2);
}
